#include<stdio.h>
#include<stdlib.h>
#include<GL/glut.h>
#include<math.h>

// Inizializzazione dello stato di OpenGL
void myInitGL()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	
}


// Funzione principale di aggiornamento dello schermo
void myRedrawFunc()
{
	int i;
	
	// Gestione dell'animazione
	float AngularVelocity = 120; // degree per sec;
	float ElapsedSecs = float(glutGet(GLUT_ELAPSED_TIME))/1000.0;
	float CurAngleDeg = AngularVelocity * ElapsedSecs;
	float CurAngleRad = CurAngleDeg/180.0*3.14159265;
	
	//Parametri dell'Anello
	const float Radius=3;
	const float BlockSide=3;
	int StepNum=24;
	float StepAngleDeg=360.0/StepNum;
	float StepAngleRad=2.0*3.14159265/StepNum;
	float InternalRotationSpeed=0.33;
	float BallSpeed=0.5;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	glOrtho(-1,1,-1,1,-1,1);

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
	
	
	// World To Camera Transformation
	glScalef(.2,.2,.2); 
	glRotatef(CurAngleDeg*0.1,1,0,0);  
	
	// -- Disegno dell'Anello -- 
	// l'anello viene create simmetrico rispetto all'asse z.
	// Per renderlo di moebius basta fare aumentare la rotazione lungo
	// l'asse y, fatta prima di ruotare intorno all'asse z; Tale rotazione 
  // deve essere la meta' di quella intorno all'asse z
	// (cosi' dopo 360 gradi ha fatto 180 gradi)
	
	glColor3f(.8,.9,.9);
	for(i=0;i<StepNum;++i)
	{
		glPushMatrix();
		glRotatef(i*StepAngleDeg    , 0,0,1);   // Rotazione Intorno all'asse principale
		glTranslatef(Radius,0,0);
		glRotatef(i*StepAngleDeg/2.0, 0,1,0);   // Avvolgimento della striscia.

		glRotatef(CurAngleDeg*InternalRotationSpeed,0,1,0);
		
		// -- Disegno del blocchetto dell'anello -- 
		glScalef(3,1,1);
		glPushMatrix();
			glTranslatef(0,0,.5);
			glutSolidCube(.5);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0,0,-.5);
			glutSolidCube(.5);
		glPopMatrix();
		glPopMatrix();
	}
	glColor3f(1,0,0);

	// -- Disegno della Pallina che rotola -- 
	// Assomiglia molto al disegno di un singolo blocchetto dell'anello.
	// a cui va aggiunta una rotazione attorno all'asse x per fare l'effetto
	// di rotolamento della pallina sulla striscia

	glRotatef(CurAngleDeg*BallSpeed,0,0,1);
	glTranslatef(Radius,0,0);
	glRotatef(CurAngleDeg*BallSpeed/2,0,1,0);
	glRotatef(CurAngleDeg*InternalRotationSpeed,0,1,0);
		
	glTranslatef(0,0,1.3);  // Per spostarla dal piano dell'anello
	glRotatef(-CurAngleDeg*BallSpeed*3,1,0,0); // Rotolamento (la quantita' e' occhio...)
	
	// -- Disegno della pallina --
	// due sfere rosse/blu leggermente scalate
	glutSolidSphere(.6,10,10);
	glColor3f(0,0,1);
	glScalef(1.05f,1.05f,.6f);
	glutSolidSphere(.6,10,10);

	glFlush();
  glutSwapBuffers();
}

void myIdle()
{
	glutPostRedisplay();
}

int main(int argc, char *argv[])
{
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInit(&argc, argv);
	glutIdleFunc(myIdle);
	glutCreateWindow("Moebius");
	glutDisplayFunc(myRedrawFunc);
	myInitGL();
	glutMainLoop(); 	//Passare al SO il controllo.
}